package dtf.core;

import stamp.core.*;

/**
 * This class defines a physical bus and provides essential I2C protocol
 * routines.  The bus pins should be pulled up to Vdd through 4.7K resistors.
 * <p>
 * This class can be used as-is, but will usually be referenced by a specific
 * device class that uses the I2C bus.  Direct use of this class requires the
 * programmer to have some understanding of the I2C bus and protocol.  Consult
 * Philips documentation for details:
 * <p>
 * http://www.semiconductors.philips.com/i2c/
 * <p>
 * Creating the bus requires two available pins: SDA (serial data) and
 * SCL (serial clock).
 * <p>
 * <code>
 *   I2C ioBus = new I2C(CPU.pin0, CPU.pin1);  // SDA on pin 0; SCL on pin 1
 * </code>
 *
 * @author Jon Williams (jwilliams@parallaxinc.com)
 * @version 1.1 15 July 2002
 */
public class I2C {

  public static final int ACK = 0x00;
  public static final int NAK = 0x01;
  int write = 0xA0;
  int read = 0xA1;
  public static Int32 num = new Int32();

  protected int sdaPin, sclPin;           // pins for I2C bus


  /**
   * Creates I2C bus object and intializes bus pins.
   *
   * @param sdaPin I2C serial data pin (bi-directional)
   * @param sclPin I2C serial clock pin
   */
  public I2C(int sdaPin, int sclPin) {
    this.sdaPin = sdaPin;
    this.sclPin = sclPin;
    CPU.setInput(sdaPin);                 // release bus pins to pull-ups
    CPU.setInput(sclPin);
  }


  /**
   * Send I2C Start sequence.
   */
  public void start() {
    CPU.setInput(sdaPin);                 // allow bus pins to be pulled high
    CPU.setInput(sclPin);
    CPU.writePin(sdaPin, false);          // take SDA low first
    while (!CPU.readPin(sclPin)) {}       // wait for clock hold to release
    CPU.writePin(sclPin, false);          // setup for high-going clock pulse
  }


  /**
   * Write 8-bit value to I2C device.
   *
   * @param i2cData Value to write to I2C device
   * @return True if device returned Ack; False if Nak
   */
  public boolean write(int block, int address, int i2cData) {
    int i2cAck;
    start();
    CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (write)<<8);
    i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
    CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (block)<<8);
    i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
    CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (address)<<8);
    i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
    CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (i2cData)<<8);
    i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
    stop();
    return (i2cAck == ACK);
  }

  public boolean write(int block, int address, StringBuffer Data) {
    int i2cAck=0;
    int track = address + Data.length();
    int character = 0;
    System.out.println(Data.toString());
    while(true)
    {
      if (address == track)
      {
        break;
      }
      char i2cData = Data.charAt(character);
      num.set(i2cData);
      num.subtract(48);
      System.out.print(num.low);
      start();
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (write)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (block)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (address)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (num.low)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      stop();
      address ++;
      character++;
      CPU.delay(500);
    }
    return (i2cAck == ACK);
  }

   /**
   * Read 8-bit value from I2C device.
   *
   * @param ackBit 0 to send Ack; 1 to send Nak after receipt of data
   * @return Value from I2C device
   */
  public int read(int block, int address, int ackBit) {
    int i2cData;
    int i2cAck;
    start();
    CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (write)<<8);
    i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
    CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (block)<<8);
    i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
    CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (address)<<8);
    i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
    stop();
    CPU.delay(50);
    start();
    CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (read)<<8);
    i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
    i2cData = CPU.shiftIn(sdaPin, sclPin, 8, CPU.PRE_CLOCK_MSB);
    stop();
    return i2cData;
  }

  public StringBuffer readstring(int block, int address, int length) {
    StringBuffer temp = new StringBuffer();
    int i2cAck;
    int i2cData;
    int count;
    temp.clear();
    for (count = 0; count < length; count ++)
    {
      start();
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (write)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (block)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (address)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      stop();
      CPU.delay(50);
      start();
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (read)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      i2cData = CPU.shiftIn(sdaPin, sclPin, 8, CPU.PRE_CLOCK_MSB);
      stop();
      temp.append(i2cData);
      CPU.delay(50);
      address++;
    }
    return temp;
  }

   public StringBuffer readdata(int block, int address, int length) {
    StringBuffer temp = new StringBuffer();
    int i2cAck;
    int i2cData;
    int count;
    for (count = 0; count < length; count ++)
    {
      start();
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (write)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (block)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (address)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      stop();
      CPU.delay(100);
      start();
      CPU.shiftOut(sdaPin, sclPin, 8, CPU.SHIFT_MSB, (read)<<8);
      i2cAck = CPU.shiftIn(sdaPin, sclPin, 1, CPU.PRE_CLOCK_MSB);
      i2cData = CPU.shiftIn(sdaPin, sclPin, 8, CPU.PRE_CLOCK_MSB);
      stop();
      temp.append(ascii.tochar(i2cData));
      System.out.println((ascii.tochar(i2cData)).toString());
      CPU.delay(100);
      address++;
    }
    return temp;
  }


  /**
   * Send I2C Stop sequence.
   */
  public void stop() {
    CPU.writePin(sdaPin, false);          // make sure SDA is low
    CPU.setInput(sclPin);                 // let SCL pin go high first
    CPU.setInput(sdaPin);
  }
}